@@ -1,9 +1,14 @@
+from argparse import Namespace
 import json
+import sys
 from collections.abc import Mapping
+from alembic.operations import Operations
+
 from .diff import Diff
 from .clause_text import get_clause_text
 import sqlalchemy as sa
 from sqlalchemy.sql.elements import TextClause
+from sqlalchemy.engine.url import make_url
 
 from alembic.migration import MigrationContext
 from alembic.autogenerate import produce_migrations
@@ -12,6 +17,8 @@ from alembic.util.exc import CommandError
 
 from sqlalchemy.dialects import postgresql
 import alembic.operations.ops as alembicops
+from alembic.operations import Operations
+from typing import Optional, Dict
 
 from . import command
 from . import config
@@ -23,14 +30,19 @@ from . import util
 
 
 class Runner(object):
-    def __init__(self, metadata, connection, schema_path):
+    def __init__(self, metadata, connection, schema_path, args: Optional[Dict] = None):
         self.metadata = metadata
         self.schema_path = schema_path
         self.connection = connection
+        self.args = args or {}
 
         config.metadata = self.metadata
         config.connection = connection
 
+        sql = self.args.get('sql', None)
+        if sql is not None and sql.lower() != 'true':
+            config.output_buffer = open(sql, 'w')
+
         self.mc = MigrationContext.configure(
             connection=self.connection,
             # note that any change here also needs a comparable change in env.py
@@ -45,11 +57,11 @@ class Runner(object):
         self.cmd = command.Command(self.connection, self.schema_path)
 
     @classmethod
-    def from_command_line(cls, metadata, args):
+    def from_command_line(cls, metadata, args: Namespace):
         engine = sa.create_engine(args.engine)
         connection = engine.connect()
         metadata.bind = connection
-        return Runner(metadata, connection, args.schema)
+        return Runner(metadata, connection, args.schema, args=args.__dict__)
 
     @classmethod
     def fix_edges(cls, metadata, args):
@@ -59,8 +71,12 @@ class Runner(object):
             engine = sa.create_engine(args.engine)
             connection = engine.connect()
 
+        mc = MigrationContext.configure(
+            connection=connection,
+        )
+        operations = Operations(mc)
         edges_map = metadata.info.setdefault("edges", {})
-        ops_impl.add_edges_from(connection, list(edges_map['public'].values()))
+        ops_impl.add_edges_from(operations, list(edges_map['public'].values()))
 
     @classmethod
     def exclude_tables(cls):
@@ -227,14 +243,14 @@ class Runner(object):
         # understand diff and make changes as needed
         # pprint.pprint(migrations, indent=2, width=30)
 
-    def upgrade(self, revision='head', merge_branches=False):
-        return self.cmd.upgrade(revision, merge_branches)
+    def upgrade(self, revision='heads', sql=False):
+        return self.cmd.upgrade(revision, sql)
 
     def downgrade(self, revision, delete_files):
         self.cmd.downgrade(revision, delete_files=delete_files)
 
-    def history(self):
-        self.cmd.history()
+    def history(self, verbose=False, last=None, rev_range=None):
+        self.cmd.history(verbose=verbose, last=last, rev_range=rev_range)
 
     def current(self):
         self.cmd.current()
@@ -264,3 +280,88 @@ class Runner(object):
 
     def squash(self, squash):
         self.cmd.squash(self.revision, squash)
+
+    # doesn't invoke env.py. completely different flow
+    # progressive_sql and upgrade range do go through offline path
+    def all_sql(self, file=None, database=''):
+        dialect = self.connection.dialect.name
+        # TODO handle
+        if dialect != 'postgresql':
+            return
+
+        raw_engine = self.args.get('engine', None)
+        if raw_engine is None:
+            return
+
+        # use passed in database. make sure not None
+        url = make_url(raw_engine).set(database=database or '')
+
+        engine = sa.create_engine(url)
+        connection = engine.connect()
+
+        metadata = sa.MetaData()
+        metadata.reflect(connection)
+        if len(metadata.sorted_tables) != 0:
+            raise Exception("to compare from base tables, cannot have any tables in database. have %d" % len(
+                metadata.sorted_tables))
+
+        mc = MigrationContext.configure(
+            connection=connection,
+            dialect_name=dialect,
+            # note that any change here also needs a comparable change in env.py
+            opts={
+                "compare_type": Runner.compare_type,
+                "include_object": Runner.include_object,
+                "compare_server_default": Runner.compare_server_default,
+                "transaction_per_migration": True,
+                "render_item": Runner.render_item,
+            },
+        )
+        migrations = produce_migrations(mc, self.metadata)
+
+        # default is stdout so let's use it
+        buffer = sys.stdout
+        if file is not None:
+            buffer = open(file, 'w')
+
+        # use different migrations context with as_sql so that we don't have issues
+        mc2 = MigrationContext.configure(
+            connection=connection,
+            # note that any change here also needs a comparable change in env.py
+            opts={
+                "compare_type": Runner.compare_type,
+                "include_object": Runner.include_object,
+                "compare_server_default": Runner.compare_server_default,
+                "render_item": Runner.render_item,
+                "as_sql": True,
+                "output_buffer": buffer,
+            },
+        )
+
+        # let's do a consistent (not runtime dependent) sort of constraints by using name instead of _creation_order
+        @property
+        def sort_constraints_by_name(self):
+            return sorted(self.constraints, key=lambda c: c.name)
+
+        sa.Table._sorted_constraints = sort_constraints_by_name
+
+        def invoke(op):
+            if isinstance(op, alembicops.OpContainer):
+                for op2 in op.ops:
+                    invoke(op2)
+            else:
+                operations.invoke(op)
+
+        operations = Operations(mc2)
+
+        # create alembic table to start
+        mc2._version.create(bind=mc2.connection)
+
+        for op in migrations.upgrade_ops.ops:
+            invoke(op)
+
+    def progressive_sql(self, file=None):
+        if file is not None:
+            config.output_buffer = open(file, 'w')
+
+        self.upgrade('base:heads', sql=True)
