@@ -1,4 +1,6 @@
 import os
+import random
+import string
 import pytest
 import shutil
 import uuid
@@ -12,37 +14,68 @@ import sqlalchemy as sa
 from auto_schema import runner
 from typing import List
 
+from auto_schema.schema_item import FullTextIndex
+from auto_schema import compare
+
 
 class Postgres:
-    def get_url(self, _schema_path):
-        return os.getenv("DB_CONNECTION_STRING", "postgresql://localhost/autoschema_test")
+    def __init__(self) -> None:
+        self._randomDB = self.randomDB()
+        self._globalConnection = None
+        self._globalEngine = None
 
-    def get_finalizer(self, metadata, session, connection, transaction, engine):
+        self._conn = None
+        self._engine = None
+
+    def randomDB(self):
+        return random.choice(string.ascii_lowercase) + ''.join(random.SystemRandom().choice(
+            string.ascii_lowercase + string.digits) for _ in range(20))
+
+    def _get_url(self, _schema_path):
+        return os.getenv("DB_CONNECTION_STRING", "postgresql://localhost")
+
+    def create_connection(self, schema_path):
+        engine = create_engine(self._get_url(schema_path),
+                               isolation_level='AUTOCOMMIT')
+        self._globalEngine = engine
+        self._globalConnection = engine.connect()
+        self._globalConnection.execute('CREATE DATABASE %s' % self._randomDB)
+        engine = create_engine("%s/%s" %
+                               (self._get_url(schema_path), self._randomDB))
+        self._engine = engine
+        self._conn = engine.connect()
+
+        return self._conn
+
+    def get_finalizer(self):
         def fn():
-            session.close()
-            metadata.reflect()
-            metadata.drop_all(bind=connection)
-            # need to commit because we sometimes commit separately because of alter type ddls
-            transaction.commit()
+            self._conn.close()
+            self._engine.dispose()
 
-            # get any newly added enum types and drop it
-            # \dT not working :(
-            for row in connection.execute("select distinct pg_type.typname as enumtype from pg_type join pg_enum on pg_enum.enumtypid = pg_type.oid;"):
-                connection.execute('drop type %s' % row['enumtype'])
+            self._globalConnection.execute('DROP DATABASE %s' % self._randomDB)
+            self._globalConnection.close()
+            self._globalEngine.dispose()
 
         return fn
 
 
 class SQLite:
-    def get_url(self, schema_path):
+
+    def __init__(self) -> None:
+        self._conn = None
+
+    def _get_url(self, schema_path):
         return "sqlite:///%s/%s" % (schema_path, "foo.db")
         # return "sqlite:///bar.db"  # if you want a local file to inspect for whatever reason
 
-    def get_finalizer(self, metadata, session, connection, transaction, engine):
+    def create_connection(self, schema_path):
+        engine = create_engine(self._get_url(schema_path))
+        self._conn = engine.connect()
+        return self._conn
+
+    def get_finalizer(self):
         def fn():
-            session.close()
-            transaction.rollback()
-            connection.close()
+            self._conn.close()
 
         return fn
 
@@ -72,14 +105,10 @@ def new_test_runner(request):
 
         # reuse connection if not None. same logic as schema_path above
         if prev_runner is None:
-            engine = create_engine(dialect.get_url(schema_path))
-            connection = engine.connect()
+            connection = dialect.create_connection(schema_path)
             metadata.bind = connection
-            transaction = connection.begin()
-            session = Session(bind=connection)
 
-            request.addfinalizer(dialect.get_finalizer(
-                metadata, session, connection, transaction, engine))
+            request.addfinalizer(dialect.get_finalizer())
         else:
             connection = prev_runner.get_connection()
 
@@ -300,11 +329,12 @@ def metadata_with_server_default_dropped(metadata):
 
 
 def metadata_with_unique_constraint_added(metadata):
-    return _add_constraint_to_metadata(
-        metadata,    # and then unique constraint added afterwards
-        sa.UniqueConstraint(
-            "email_address", name="accounts_unique_email_address"),
-    )
+    sa.Table('accounts', metadata,
+             sa.UniqueConstraint(
+                 "email_address", name="accounts_unique_email_address"),
+             extend_existing=True
+             )
+    return metadata
 
 
 # takes the account table and converts the email_address type from String(255) to Text()
@@ -376,20 +406,125 @@ def _apply_func_on_metadata(metadata, col_name, table_name, fn):
 
 
 def metadata_with_table_with_index(metadata_with_table):
-    return _add_constraint_to_metadata(
-        metadata_with_table,
-        # index the first name because we support searching for some reason
-        sa.Index("accounts_first_name_idx", "first_name"),
-    )
+    sa.Table('accounts',
+             metadata_with_table,
+             # index the first name because we support searching for some reason
+             sa.Index("accounts_first_name_idx", "first_name"),
+             extend_existing=True
+             )
+    return metadata_with_table
 
 
 def metadata_with_multi_column_index(metadata_with_table):
-    return _add_constraint_to_metadata(
-        metadata_with_table,
-        # index the first and last name because we support searching by that
-        sa.Index("accounts_first_name_last_name_idx",
-                 "first_name", "last_name"),
-    )
+    sa.Table('accounts',
+             metadata_with_table,
+             # index the first and last name because we support searching by that
+             sa.Index("accounts_first_name_last_name_idx",
+                      "first_name", "last_name"),
+             extend_existing=True
+             )
+    return metadata_with_table
+
+
+def metadata_with_fulltext_search_index(metadata_with_table):
+    sa.Table('accounts',
+             metadata_with_table,
+             FullTextIndex("accounts_first_name_idx",
+                           info={
+                               'postgresql_using': 'gin',
+                               'postgresql_using_internals': "to_tsvector('english', first_name)",
+                               'column': 'first_name',
+                           }
+                           ),
+             extend_existing=True
+             )
+    return metadata_with_table
+
+
+def metadata_with_multicolumn_fulltext_search_index(metadata_with_table):
+    sa.Table('accounts',
+             metadata_with_table,
+             FullTextIndex("accounts_full_text_idx",
+                           info={
+                               'postgresql_using': 'gin',
+                               'postgresql_using_internals': "to_tsvector('english', first_name || ' ' || last_name)",
+                               'columns': ['first_name', 'last_name'],
+                           }
+                           ),
+             extend_existing=True
+             )
+    return metadata_with_table
+
+
+@pytest.fixture
+def metadata_with_multicolumn_fulltext_search():
+    metadata = metadata_with_base_table_restored()
+    sa.Table('accounts',
+             metadata,
+             FullTextIndex("accounts_full_text_idx",
+                           info={
+                               'postgresql_using': 'gin',
+                               'postgresql_using_internals': "to_tsvector('english', first_name || ' ' || last_name)",
+                               'columns': ['first_name', 'last_name'],
+                           }
+                           ),
+             extend_existing=True
+             )
+    return metadata
+
+
+def metadata_with_multicolumn_fulltext_search_index_gist(metadata_with_table):
+    sa.Table('accounts',
+             metadata_with_table,
+             FullTextIndex("accounts_full_text_idx",
+                           info={
+                               'postgresql_using': 'gist',
+                               'postgresql_using_internals': "to_tsvector('english', first_name || ' ' || last_name)",
+                               'columns': ['first_name', 'last_name'],
+                           }
+                           ),
+             extend_existing=True
+             )
+    return metadata_with_table
+
+
+def metadata_with_multicolumn_fulltext_search_index_btree(metadata_with_table):
+    sa.Table('accounts',
+             metadata_with_table,
+             FullTextIndex("accounts_full_text_idx",
+                           info={
+                               'postgresql_using': 'btree',
+                               'postgresql_using_internals': "to_tsvector('english', first_name || ' ' || last_name)",
+                               'columns': ['first_name', 'last_name'],
+                           }
+                           ),
+             extend_existing=True
+             )
+    return metadata_with_table
+
+
+def metadata_with_generated_col_fulltext_search_index(metadata_with_table):
+    sa.Table('accounts', metadata_with_table,
+             sa.Column('full_name', postgresql.TSVECTOR(), sa.Computed(
+                 "to_tsvector('english', first_name || ' ' || last_name)")),
+             sa.Index('accounts_full_text_idx',
+                      'full_name', postgresql_using='gin'),
+
+             extend_existing=True)
+
+    return metadata_with_table
+
+
+def metadata_with_generated_col_fulltext_search_index_gist(metadata_with_table):
+    sa.Table('accounts', metadata_with_table,
+             sa.Column('full_name', postgresql.TSVECTOR(), sa.Computed(
+                 "to_tsvector('english', first_name || ' ' || last_name)")),
+             sa.Index('accounts_full_text_idx',
+                      'full_name', postgresql_using='gist'),
+
+             extend_existing=True)
+
+    return metadata_with_table
 
 
 @ pytest.fixture
@@ -416,11 +551,13 @@ def metadata_with_multi_column_pkey_constraint(request):
 @ pytest.fixture()
 def metadata_with_multi_column_unique_constraint():
     metadata = metadata_with_contacts_table_with_no_unique_constraint()
-    return _add_constraint_to_metadata(metadata,
-                                       sa.UniqueConstraint(
-                                           "email_address", "user_id", name="contacts_unique_email_per_contact"
-                                       ), table_name='contacts'
-                                       )
+    sa.Table('contacts', metadata,
+             sa.UniqueConstraint(
+                 "email_address", "user_id", name="contacts_unique_email_per_contact"
+             ),
+             extend_existing=True
+             )
+    return metadata
 
 
 def metadata_with_contacts_table_with_no_unique_constraint():
@@ -534,10 +671,13 @@ def metadata_with_column_check_constraint():
 
 
 def metadata_with_constraint_added_after(metadata):
-    return _add_constraint_to_metadata(
-        metadata,
-        sa.CheckConstraint('meaning_of_life = 42', 'meaning_of_life_correct'),
-    )
+    sa.Table('accounts',
+             metadata,
+             sa.CheckConstraint('meaning_of_life = 42',
+                                'meaning_of_life_correct'),
+             extend_existing=True
+             )
+    return metadata
 
 
 @ pytest.fixture()
@@ -587,7 +727,7 @@ def _metadata_assoc_edge_config(request):
     return metadata
 
 
-@pytest.fixture
+@ pytest.fixture
 def metadata_with_assoc_edge_config(request):
     return _metadata_assoc_edge_config(request)
 
@@ -1039,11 +1179,3 @@ def assoc_edge_config_table(metadata, request):
              sa.ForeignKeyConstraint(['inverse_edge_type'], ['assoc_edge_config.edge_type'],
                                      name="assoc_edge_config_inverse_edge_type_fkey", ondelete="RESTRICT"),
              )
-
-
-def _add_constraint_to_metadata(metadata, constraint, table_name="accounts"):
-    tables = [t for t in metadata.sorted_tables if t.name == table_name]
-    table = tables[0]
-
-    table.append_constraint(constraint)
-    return metadata
